@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2000, 2014, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -30,7 +30,6 @@
 #include "driver.h"
 #include "installer.h"
 #include "stringutil.h"
-#include "MYODBCUtil.h"
 
 #ifndef CLIENT_NO_SCHEMA
 # define CLIENT_NO_SCHEMA      16
@@ -84,6 +83,15 @@ SQLRETURN myodbc_set_initial_character_set(DBC *dbc, const char *charset)
       dbc->ansi_charset_info= get_charset_by_csname(charset,
                                                     MYF(MY_CS_PRIMARY),
                                                     MYF(0));
+      if (!dbc->ansi_charset_info)
+      {
+        char errmsg[NAME_LEN + 32*SYSTEM_CHARSET_MBMAXLEN];
+        /* This message should help the user to identify the error */
+        sprintf(errmsg, "Wrong character set name %.*s", NAME_LEN, charset);
+        set_dbc_error(dbc, "HY000", errmsg, 0);
+
+        return SQL_ERROR;
+      }
     }
 
     charset= "utf8";
@@ -340,8 +348,11 @@ SQLRETURN myodbc_do_connect(DBC *dbc, DataSource *ds)
   ds_get_utf8attr(ds->pwd, &ds->pwd8);
   ds_get_utf8attr(ds->socket, &ds->socket8);
   if (ds->database)
+  {
+    x_free(dbc->database);
     dbc->database= my_strdup(ds_get_utf8attr(ds->database, &ds->database8),
                              MYF(MY_WME));
+  }
   
   if (ds->save_queries && !dbc->query_log)
     dbc->query_log= init_query_log();
@@ -800,6 +811,8 @@ SQLRETURN SQL_API SQLDisconnect(SQLHDBC hdbc)
   LIST *list_element, *next_element;
   DBC *dbc= (DBC *) hdbc;
 
+  CHECK_HANDLE(hdbc);
+
   for (list_element= dbc->statements; list_element; list_element= next_element)
   {
      next_element= list_element->next;
@@ -810,6 +823,12 @@ SQLRETURN SQL_API SQLDisconnect(SQLHDBC hdbc)
   if (dbc->ds->save_queries)
     end_query_log(dbc->query_log);
 
+  /* free allocated packet buffer */
+  if (dbc->mysql.net.buff)
+  {
+    myodbc_net_end(&dbc->mysql.net);
+  }
+
   x_free(dbc->database);
   assert(dbc->ds);
   ds_delete(dbc->ds);
