@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2007, 2012, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -266,6 +266,11 @@ void desc_remove_stmt(DESC *desc, STMT *stmt)
     if (lstmt->data == stmt)
     {
       desc->exp.stmts= list_delete(desc->exp.stmts, lstmt);
+      /* Free only if it was the last element */
+      if(!lstmt->next && !lstmt->prev)
+      {
+        x_free(lstmt);
+      }
       return;
     }
   }
@@ -518,6 +523,11 @@ MySQLGetDescField(SQLHDESC hdesc, SQLSMALLINT recnum, SQLSMALLINT fldid,
   void *src_struct;
   void *src;
 
+  if (desc == NULL)
+  {
+    return SQL_INVALID_HANDLE;
+  }
+
   CLEAR_DESC_ERROR(desc);
 
   if (IS_IRD(desc) && desc->stmt->state < ST_PREPARED)
@@ -698,6 +708,11 @@ MySQLSetDescField(SQLHDESC hdesc, SQLSMALLINT recnum, SQLSMALLINT fldid,
   void *dest_struct;
   void *dest;
 
+  if (desc == NULL)
+  {
+    return SQL_INVALID_HANDLE;
+  }
+
   CLEAR_DESC_ERROR(desc);
 
   /* check for invalid IRD modification */
@@ -965,6 +980,9 @@ SQLRETURN stmt_SQLCopyDesc(STMT *stmt, DESC *src, DESC *dest)
 SQLRETURN SQL_API
 SQLCopyDesc(SQLHDESC SourceDescHandle, SQLHDESC TargetDescHandle)
 {
+  CHECK_HANDLE(SourceDescHandle);
+  CHECK_HANDLE(TargetDescHandle);
+
   return MySQLCopyDesc(SourceDescHandle, TargetDescHandle);
 }
 
