@@ -1,6 +1,5 @@
 /*
-  Copyright (c) 2001, 2007 MySQL AB, 2010 Sun Microsystems, Inc.
-  Use is subject to license terms.
+  Copyright (c) 2001, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -37,6 +36,8 @@
 #ifndef __ERROR_H__
 #define __ERROR_H__
 
+/* Including driver version definitions */
+#include "../MYODBC_CONF.h"
 /*
   myodbc internal error constants
 */
@@ -44,9 +45,9 @@
 #define ER_ALL_COLUMNS_IGNORED	 537
 
 /*
-  myodbc3 error prefix
+  myodbc5 error prefix
 */
-#define MYODBC3_ERROR_PREFIX	 "[MySQL][ODBC 5.1 Driver]"
+#define MYODBC_ERROR_PREFIX	 "[MySQL][ODBC "MYODBC_STRDRIVERID" Driver]"
 #define MYODBC_ERROR_CODE_START  500
 
 /*
