@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2000, 2014, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -42,13 +42,14 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
 
     if (!query)
     {
-      return error;       /* Probably error from insert_param */
+      /* Probably error from insert_param */
+      goto skip_unlock_exit;
     }
 
     if(!SQL_SUCCEEDED(set_sql_select_limit(stmt->dbc, stmt->stmt_options.max_rows)))
     {
       /* if setting sql_select_limit fails, the query will probably fail anyway too */
-      return error;
+      goto skip_unlock_exit;
     }
 
     if (query_length == 0)
@@ -186,8 +187,10 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length)
 
     error= SQL_SUCCESS;
 
-    exit:
+exit:
     pthread_mutex_unlock(&stmt->dbc->lock);
+
+skip_unlock_exit:
     if (query != GET_QUERY(&stmt->query))
     {
       x_free(query);
@@ -1186,7 +1189,9 @@ SQLRETURN do_my_pos_cursor( STMT *pStmt, STMT *pStmtCursor )
 
 SQLRETURN SQL_API SQLExecute(SQLHSTMT hstmt)
 {
-    return my_SQLExecute((STMT *)hstmt);
+  CHECK_HANDLE(hstmt);
+
+  return my_SQLExecute((STMT *)hstmt);
 }
 
 
@@ -1432,6 +1437,21 @@ SQLRETURN my_SQLExecute( STMT *pStmt )
       }
       else
       {
+        if (query != GET_QUERY(&pStmt->query))
+        {
+          x_free(query);
+        }
+  
+        /*
+          If the original query was modified, we reset stmt->query so that the
+          next execution re-starts with the original query.
+        */
+        if (GET_QUERY(&pStmt->orig_query))
+        {
+          copy_parsed_query(&pStmt->orig_query, &pStmt->query);
+          reset_parsed_query(&pStmt->orig_query, NULL, NULL, NULL);
+        }
+
         /* with broken connection we always return error for all next queries */
         rc= SQL_ERROR;
       }
@@ -1515,9 +1535,15 @@ SQLRETURN SQL_API SQLParamData(SQLHSTMT hstmt, SQLPOINTER *prbgValue)
     STMT *stmt= (STMT *) hstmt;
     uint i;
     SQLRETURN rc;
-    char *query=  GET_QUERY(&stmt->query);
+    char *query;
     DESC *apd;
-    uint param_count= stmt->param_count;
+    uint param_count;
+
+    /* We only check hstmt, no need to check prbgValue */
+    CHECK_HANDLE(hstmt);
+
+    query=  GET_QUERY(&stmt->query);
+    param_count= stmt->param_count;
 
     assert(stmt->dae_type);
     /* get the correct APD for the dae type we're handling */
@@ -1607,16 +1633,28 @@ SQLRETURN SQL_API SQLPutData( SQLHSTMT      hstmt,
     STMT *stmt= (STMT *) hstmt;
     DESCREC *aprec;
 
-    if ( !stmt )
-        return SQL_ERROR;
+    CHECK_HANDLE(hstmt);
+    CHECK_DATA_POINTER(stmt, rgbValue, cbValue);
+    CHECK_STRLEN_OR_IND(stmt, rgbValue, cbValue);
 
-    if ( cbValue == SQL_NTS )
-        cbValue= strlen(rgbValue);
     if (stmt->dae_type == DAE_NORMAL)
       aprec= desc_get_rec(stmt->apd, stmt->current_param - 1, FALSE);
     else
       aprec= desc_get_rec(stmt->setpos_apd, stmt->current_param - 1, FALSE);
     assert(aprec);
+
+    if ( cbValue == SQL_NTS )
+    {
+      if (aprec->concise_type == SQL_C_WCHAR)
+      {
+        cbValue= sqlwcharlen((SQLWCHAR *)rgbValue) * sizeof(SQLWCHAR);
+      }
+      else
+      {
+        cbValue= strlen(rgbValue);
+      }
+    }
+
     if ( cbValue == SQL_NULL_DATA )
     {
         if ( aprec->par.alloced )
@@ -1666,10 +1704,13 @@ SQLRETURN SQL_API SQLPutData( SQLHSTMT      hstmt,
 */
 SQLRETURN SQL_API SQLCancel(SQLHSTMT hstmt)
 {
-  DBC *dbc= ((STMT *)hstmt)->dbc;
   MYSQL *second= NULL;
   int error;
+  DBC *dbc;
 
+  CHECK_HANDLE(hstmt);
+
+  dbc= ((STMT *)hstmt)->dbc;
   error= pthread_mutex_trylock(&dbc->lock);
 
   /* If there's no query going on, just close the statement. */
