@@ -87,7 +87,15 @@ SQLRETURN SQL_API my_SQLAllocEnv(SQLHENV *phenv)
     }
 #endif /* _UNIX_ */
     pthread_mutex_init(&(*env)->lock,NULL);
-    return SQL_SUCCESS;
+
+/* --- if OS=WIN32, set default env option for SQL_ATTR_ODBC_VERSION */
+#ifdef WIN32
+    ((ENV *) *phenv)->odbc_ver= SQL_OV_ODBC3;
+#else
+    ((ENV *) *phenv)->odbc_ver= SQL_OV_ODBC2;
+#endif /* WIN32 */
+ 
+   return SQL_SUCCESS;
 }
 
 
@@ -98,20 +106,9 @@ SQLRETURN SQL_API my_SQLAllocEnv(SQLHENV *phenv)
 
 SQLRETURN SQL_API SQLAllocEnv(SQLHENV *phenv)
 {
-  SQLRETURN rc;
+  CHECK_ENV_OUTPUT(phenv);
 
-  rc= my_SQLAllocEnv(phenv);
-  if (rc == SQL_SUCCESS)
-  {
-/* --- if OS=WIN32, set default env option for SQL_ATTR_ODBC_VERSION */
-#ifdef WIN32
-    ((ENV *) *phenv)->odbc_ver= SQL_OV_ODBC3;
-#else
-    ((ENV *) *phenv)->odbc_ver= SQL_OV_ODBC2;
-#endif /* WIN32 */
-  }
-
-  return rc;
+  return my_SQLAllocEnv(phenv);
 }
 
 
@@ -141,6 +138,8 @@ SQLRETURN SQL_API my_SQLFreeEnv(SQLHENV henv)
 */
 SQLRETURN SQL_API SQLFreeEnv(SQLHENV henv)
 {
+    CHECK_HANDLE(henv);
+
     return my_SQLFreeEnv(henv);
 }
 
@@ -272,7 +271,11 @@ SQLRETURN SQL_API my_SQLAllocConnect(SQLHENV henv, SQLHDBC *phdbc)
 
 SQLRETURN SQL_API SQLAllocConnect(SQLHENV henv, SQLHDBC *phdbc)
 {
-    return my_SQLAllocConnect(henv, phdbc);
+  /* Checking only henv because phdbc will be checked later */
+  CHECK_HANDLE(henv);
+  CHECK_DBC_OUTPUT(henv, phdbc);
+
+  return my_SQLAllocConnect(henv, phdbc);
 }
 
 
@@ -345,7 +348,9 @@ SQLRETURN SQL_API my_SQLFreeConnect(SQLHDBC hdbc)
 */
 SQLRETURN SQL_API SQLFreeConnect(SQLHDBC hdbc)
 {
-    return my_SQLFreeConnect(hdbc);
+  CHECK_HANDLE(hdbc);
+
+  return my_SQLFreeConnect(hdbc);
 }
 
 /* allocates dynamic array for param bind.
@@ -471,6 +476,9 @@ error:
 
 SQLRETURN SQL_API SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT *phstmt)
 {
+  CHECK_HANDLE(hdbc);
+  CHECK_STMT_OUTPUT(hdbc, phstmt);
+
     return my_SQLAllocStmt(hdbc,phstmt);
 }
 
@@ -485,6 +493,8 @@ SQLRETURN SQL_API SQLAllocStmt(SQLHDBC hdbc,SQLHSTMT *phstmt)
 
 SQLRETURN SQL_API SQLFreeStmt(SQLHSTMT hstmt,SQLUSMALLINT fOption)
 {
+    CHECK_HANDLE(hstmt);
+
     return my_SQLFreeStmt(hstmt,fOption);
 }
 
@@ -565,6 +575,7 @@ SQLRETURN SQL_API my_SQLFreeStmtExtended(SQLHSTMT hstmt,SQLUSMALLINT fOption,
       {
         free_root(&stmt->result->field_alloc, MYF(0));
       }
+
       x_free(stmt->result);
     }
 
@@ -761,18 +772,25 @@ SQLRETURN SQL_API SQLAllocHandle(SQLSMALLINT HandleType,
     switch (HandleType)
     {
         case SQL_HANDLE_ENV:
+            CHECK_ENV_OUTPUT(OutputHandlePtr);
             error= my_SQLAllocEnv(OutputHandlePtr);
             break;
 
         case SQL_HANDLE_DBC:
+            CHECK_HANDLE(InputHandle);
+            CHECK_DBC_OUTPUT(InputHandle, OutputHandlePtr);
             error= my_SQLAllocConnect(InputHandle,OutputHandlePtr);
             break;
 
         case SQL_HANDLE_STMT:
+            CHECK_HANDLE(InputHandle);
+            CHECK_STMT_OUTPUT(InputHandle, OutputHandlePtr);
             error= my_SQLAllocStmt(InputHandle,OutputHandlePtr);
             break;
 
         case SQL_HANDLE_DESC:
+            CHECK_HANDLE(InputHandle);
+            CHECK_DESC_OUTPUT(InputHandle, OutputHandlePtr);
             error= my_SQLAllocDesc(InputHandle, OutputHandlePtr);
             break;
 
@@ -795,6 +813,8 @@ SQLRETURN SQL_API SQLFreeHandle(SQLSMALLINT HandleType,
 {
     SQLRETURN error= SQL_ERROR;
 
+    CHECK_HANDLE(Handle);
+
     switch (HandleType)
     {
         case SQL_HANDLE_ENV:
