@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2012-2013, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -125,7 +125,9 @@ BOOL ssps_get_out_params(STMT *stmt)
 
     if (values != NULL && got_out_parameters(stmt))
     {
-      for (i= 0; i < myodbc_min(stmt->ipd->count, stmt->apd->count); ++i)
+      for (i= 0;
+           i < myodbc_min(stmt->ipd->count, stmt->apd->count) && counter < field_count(stmt);
+           ++i)
       {
         /* Making bit field look "normally" */
         if (stmt->result_bind[counter].buffer_type == MYSQL_TYPE_BIT)
@@ -233,6 +235,11 @@ void free_result_bind(STMT *stmt)
     for (i= 0; i < field_cnt; i++)
     {
       x_free(stmt->result_bind[i].buffer);
+
+      if (stmt->lengths)
+      {
+        stmt->lengths[i]= 0;
+      }
     }
 
     x_free(stmt->result_bind);
@@ -416,6 +423,12 @@ int ssps_bind_result(STMT *stmt)
         /* length marks such fields */
         if (stmt->lengths[i] > 0)
         {
+          if (stmt->result_bind[i].buffer == stmt->array[i])
+          {
+            /* make sure we do not free it twice */
+            stmt->array[i]= NULL;
+            stmt->lengths[i]= 0;
+          }
           /* Resetting buffer and buffer_length for those fields */
           stmt->result_bind[i].buffer       = 0;
           stmt->result_bind[i].buffer_length= 0;
