@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2012, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2012, 2013, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -65,7 +65,7 @@ BOOL returned_result(STMT *stmt)
 my_bool free_current_result(STMT *stmt)
 {
   my_bool res= 0;
-  if (returned_result(stmt))
+  if (stmt->result)
   {
     if (ssps_used(stmt))
     {
