@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2001, 2013, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2001, 2014, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -138,7 +138,7 @@ SQLRETURN do_query(STMT *stmt,char *query, SQLULEN query_length);
 SQLRETURN insert_params(STMT *stmt, SQLULEN row, char **finalquery,
                         SQLULEN *length);
 SQLRETURN odbc_stmt(DBC *dbc, const char *query);
-void mysql_link_fields(STMT *stmt,MYSQL_FIELD *fields,uint field_count);
+void myodbc_link_fields(STMT *stmt,MYSQL_FIELD *fields,uint field_count);
 void fix_row_lengths(STMT *stmt, const long* fix_rules, uint row, uint field_count);
 void fix_result_types(STMT *stmt);
 char *fix_str(char *to,const char *from,int length);
@@ -261,6 +261,8 @@ SQLRETURN SQL_API my_SQLFreeConnect(SQLHDBC hdbc);
 SQLRETURN SQL_API my_SQLFreeEnv(SQLHENV henv);
 char *extend_buffer(NET *net,char *to,ulong length);
 void myodbc_end();
+my_bool myodbc_net_realloc(NET *net, size_t length);
+void myodbc_net_end(NET *net);
 my_bool set_dynamic_result(STMT *stmt);
 void set_current_cursor_data(STMT *stmt,SQLUINTEGER irow);
 my_bool is_minimum_version(const char *server_version,const char *version);
@@ -338,6 +340,8 @@ MY_LIMIT_CLAUSE find_position4limit(CHARSET_INFO* cs, char *query,
                                     char * query_end);
 BOOL myodbc_isspace(CHARSET_INFO* cs, const char * begin, const char *end);
 BOOL got_out_parameters(STMT *stmt);
+const char get_identifier_quote(STMT *stmt);
+int get_session_variable(STMT *stmt, const char *var, char *result);
 
 /* handle.c*/
 BOOL          allocate_param_bind(DYNAMIC_ARRAY **param_bind, uint elements);
@@ -425,4 +429,29 @@ char *      ssps_get_string       (STMT *stmt, ulong column_number, char *value,
                             (*(X) == SQL_DATA_AT_EXEC || \
                              *(X) <= SQL_LEN_DATA_AT_EXEC_OFFSET))
 
+#define GET_NAME_LEN(S, N, L) L = (L == SQL_NTS ? (N ? (SQLSMALLINT)strlen((char *)N) : 0) : L); \
+  if (L > NAME_LEN) \
+    return set_stmt_error(S, "HY090", \
+           "One or more parameters exceed the maximum allowed name length", 0);
+
+#define CHECK_HANDLE(h) if (h == NULL) return SQL_INVALID_HANDLE
+
+#define CHECK_DATA_POINTER(S, D, C) if (D == NULL && C != 0 && C != SQL_DEFAULT_PARAM && C != SQL_NULL_DATA) \
+                                   return set_stmt_error(S, "HY009", "Invalid use of NULL pointer", 0);
+
+#define CHECK_STRLEN_OR_IND(S, D, C) if (D != NULL && C < 0 && C != SQL_NTS && C != SQL_NULL_DATA) \
+                                   return set_stmt_error(S, "HY090", "Invalid string or buffer length", 0);
+
+#define CHECK_ENV_OUTPUT(e) if(e == NULL) return SQL_ERROR
+
+#define CHECK_DBC_OUTPUT(e, d) if(d == NULL) return set_env_error((ENV *)e, MYERR_S1009, NULL, 0)
+
+#define CHECK_STMT_OUTPUT(d, s) if(s == NULL) return set_conn_error((DBC *)d, MYERR_S1009, NULL, 0)
+ 
+#define CHECK_DESC_OUTPUT(d, s) CHECK_STMT_OUTPUT(d, s)
+
+#define CHECK_DATA_OUTPUT(s, d) if(d == NULL) return set_error((STMT *)s, MYERR_S1000, "Invalid output buffer", 0)
+ 
+#define IF_NOT_NULL(v, x) if (v != NULL) x
+
 #endif /* __MYUTIL_H__ */
