@@ -287,6 +287,14 @@ MySQLSetConnectAttr(SQLHDBC hdbc, SQLINTEGER Attribute,
         case SQL_ATTR_CURRENT_CATALOG:
             {
                 char ldb[NAME_LEN+1], *db;
+                int cat_len= StringLengthPtr == SQL_NTS ? 
+                             strlen((char *)ValuePtr) : StringLengthPtr;
+
+                if (cat_len > NAME_LEN)
+                {
+                  return set_conn_error(hdbc, MYERR_01004, 
+                                        "Invalid string or buffer length", 0);
+                }
 
                 if (!(db= fix_str((char *)ldb, (char *)ValuePtr, StringLengthPtr)))
                     return set_conn_error(hdbc,MYERR_S1009,NULL, 0);
@@ -597,12 +605,15 @@ MySQLSetStmtAttr(SQLHSTMT hstmt, SQLINTEGER Attribute, SQLPOINTER ValuePtr,
 
               if (desc->alloc_type == SQL_DESC_ALLOC_AUTO &&
                   (*dest)->alloc_type == SQL_DESC_ALLOC_USER)
+              {
                 /*
                   If we're setting back the original implicit
                   descriptor, we must disassociate this statement
                   from the explicit descriptor.
                 */
                 desc_remove_stmt(*dest, stmt);
+                
+              }
               else if (desc->alloc_type == SQL_DESC_ALLOC_USER)
               {
                 /* otherwise, associate this statement with the desc */
@@ -850,6 +861,7 @@ SQLSetEnvAttr(SQLHENV    henv,
               SQLPOINTER ValuePtr,
               SQLINTEGER StringLength __attribute__((unused)))
 {
+    CHECK_HANDLE(henv);
 
     if (((ENV *)henv)->connections)
         return set_env_error(henv, MYERR_S1010, NULL, 0);
@@ -883,18 +895,21 @@ SQLGetEnvAttr(SQLHENV    henv,
               SQLINTEGER BufferLength __attribute__((unused)),
               SQLINTEGER *StringLengthPtr __attribute__((unused)))
 {
+    CHECK_HANDLE(henv);
+    /* NULL is acceptable for ValuePtr, so we are not checking for it here */
+
     switch ( Attribute )
     {
         case SQL_ATTR_CONNECTION_POOLING:
-            *(SQLINTEGER*)ValuePtr = SQL_CP_OFF;
+            IF_NOT_NULL(ValuePtr, *(SQLINTEGER*)ValuePtr = SQL_CP_OFF);
             break;
 
         case SQL_ATTR_ODBC_VERSION:
-            *(SQLINTEGER*)ValuePtr= ((ENV *)henv)->odbc_ver;
+            IF_NOT_NULL(ValuePtr, *(SQLINTEGER*)ValuePtr= ((ENV *)henv)->odbc_ver);
             break;
 
         case SQL_ATTR_OUTPUT_NTS:
-            *((SQLINTEGER*)ValuePtr)= SQL_TRUE;
+            IF_NOT_NULL(ValuePtr, *((SQLINTEGER*)ValuePtr)= SQL_TRUE);
             break;
 
         default:
@@ -907,6 +922,8 @@ SQLGetEnvAttr(SQLHENV    henv,
 SQLRETURN SQL_API
 SQLGetStmtOption(SQLHSTMT hstmt,SQLUSMALLINT option, SQLPOINTER param)
 {
+  CHECK_HANDLE(hstmt);
+
   return MySQLGetStmtAttr(hstmt, option, param, SQL_NTS, (SQLINTEGER *)NULL);
 }
 
@@ -914,5 +931,7 @@ SQLGetStmtOption(SQLHSTMT hstmt,SQLUSMALLINT option, SQLPOINTER param)
 SQLRETURN SQL_API
 SQLSetStmtOption(SQLHSTMT hstmt, SQLUSMALLINT option, SQLULEN param)
 {
+  CHECK_HANDLE(hstmt);
+
   return MySQLSetStmtAttr(hstmt, option, (SQLPOINTER)param, SQL_NTS);
 }
