@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2000, 2014, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -759,6 +759,9 @@ SQLRETURN SQL_API SQLNumResultCols(SQLHSTMT  hstmt, SQLSMALLINT *pccol)
   SQLRETURN error;
   STMT *stmt= (STMT *) hstmt;
 
+  CHECK_HANDLE(hstmt);
+  CHECK_DATA_OUTPUT(hstmt, pccol);
+
   if (!ssps_used(stmt))
   {
     if (stmt->param_count > 0 && stmt->dummy_state == ST_DUMMY_UNKNOWN &&
@@ -802,6 +805,8 @@ MySQLDescribeCol(SQLHSTMT hstmt, SQLUSMALLINT column,
   STMT *stmt= (STMT *)hstmt;
   DESCREC* irrec;
 
+  *need_free= 0;
+
   /* SQLDescribeCol can be called before SQLExecute. Thus we need make sure that
      all parameters have been bound */
   if (!ssps_used(stmt))
@@ -817,8 +822,11 @@ MySQLDescribeCol(SQLHSTMT hstmt, SQLUSMALLINT column,
       return error;
     if (!stmt->result)
       return set_stmt_error(stmt, "07005", "No result set", 0);
-    if (column == 0 || column > stmt->ird->count)
-      return set_stmt_error(stmt, "07009", "Invalid descriptor index", 0);
+  }
+
+  if (column == 0 || column > stmt->ird->count)
+  {
+    return set_stmt_error(stmt, "07009", "Invalid descriptor index", 0);
   }
 
   irrec= desc_get_rec(stmt->ird, column - 1, FALSE);
@@ -833,8 +841,6 @@ MySQLDescribeCol(SQLHSTMT hstmt, SQLUSMALLINT column,
   if (nullable)
     *nullable= irrec->nullable;
 
-  *need_free= 0;
-
   if (stmt->dbc->ds->return_table_names_for_SqlDescribeCol && irrec->table_name)
   {
     char *tmp= my_malloc(strlen((char *)irrec->name) +
@@ -1059,6 +1065,8 @@ SQLRETURN SQL_API SQLBindCol(SQLHSTMT      StatementHandle,
   DESCREC *arrec;
   /* TODO if this function fails, the SQL_DESC_COUNT should be unchanged in ard */
 
+  CHECK_HANDLE(stmt);
+
   CLEAR_STMT_ERROR(stmt);
 
   if (!TargetValuePtr && !StrLen_or_IndPtr) /* Handling unbinding */
@@ -1171,6 +1179,8 @@ SQLRETURN SQL_API SQLGetData(SQLHSTMT      StatementHandle,
     ulong length= 0;
     DESCREC *irrec;
 
+    CHECK_HANDLE(stmt);
+
     if (!stmt->result || !stmt->current_values)
     {
       set_stmt_error(stmt,"24000","SQLGetData without a preceding SELECT",0);
@@ -1226,6 +1236,8 @@ SQLRETURN SQL_API SQLMoreResults( SQLHSTMT hStmt )
   int         nRetVal;
   SQLRETURN   nReturn = SQL_SUCCESS;
 
+  CHECK_HANDLE(hStmt);
+
   pthread_mutex_lock( &pStmt->dbc->lock );
 
   CLEAR_STMT_ERROR( pStmt );
@@ -1338,6 +1350,9 @@ SQLRETURN SQL_API SQLRowCount( SQLHSTMT hstmt,
 {
     STMT *stmt= (STMT *) hstmt;
 
+    CHECK_HANDLE(hstmt);
+    CHECK_DATA_OUTPUT(hstmt, pcrow);
+
     if ( stmt->result )
     {
       /* for SetPos operations result is defined and they use direct execution */
@@ -1843,8 +1858,11 @@ SQLRETURN SQL_API SQLExtendedFetch( SQLHSTMT        hstmt,
 {
     SQLRETURN rc;
     SQLULEN rows;
-    STMT_OPTIONS *options= &((STMT *)hstmt)->stmt_options;
+    STMT_OPTIONS *options;
 
+    CHECK_HANDLE(hstmt);
+
+    options= &((STMT *)hstmt)->stmt_options;
     options->rowStatusPtr_ex= rgfRowStatus;
 
     rc= my_SQLExtendedFetch(hstmt, fFetchType, irow, &rows, rgfRowStatus, 1);
@@ -1867,8 +1885,11 @@ SQLRETURN SQL_API SQLFetchScroll( SQLHSTMT      StatementHandle,
                                   SQLLEN        FetchOffset )
 {
     STMT *stmt = (STMT *)StatementHandle;
-    STMT_OPTIONS *options= &stmt->stmt_options;
+    STMT_OPTIONS *options;
 
+    CHECK_HANDLE(stmt);
+
+    options= &stmt->stmt_options;
     options->rowStatusPtr_ex= NULL;
 
     return my_SQLExtendedFetch(StatementHandle, FetchOrientation, FetchOffset,
@@ -1885,8 +1906,11 @@ SQLRETURN SQL_API SQLFetchScroll( SQLHSTMT      StatementHandle,
 SQLRETURN SQL_API SQLFetch(SQLHSTMT StatementHandle)
 {
     STMT *stmt = (STMT *)StatementHandle;
-    STMT_OPTIONS *options= &stmt->stmt_options;
+    STMT_OPTIONS *options;
 
+    CHECK_HANDLE(stmt);
+
+    options= &stmt->stmt_options;
     options->rowStatusPtr_ex= NULL;
 
     return my_SQLExtendedFetch(StatementHandle, SQL_FETCH_NEXT, 0,
