@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2000, 2013, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2000, 2014, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -68,7 +68,7 @@ SQLRETURN odbc_stmt(DBC *dbc, const char *query)
   @param[in] fields      The fields to attach to the statement
   @param[in] field_count The number of fields
 */
-void mysql_link_fields(STMT *stmt, MYSQL_FIELD *fields, uint field_count)
+void myodbc_link_fields(STMT *stmt, MYSQL_FIELD *fields, uint field_count)
 {
     MYSQL_RES *result;
     pthread_mutex_lock(&stmt->dbc->lock);
@@ -82,7 +82,7 @@ void mysql_link_fields(STMT *stmt, MYSQL_FIELD *fields, uint field_count)
 
 
 /**
-Fills STMT's lengths array for given row. Makes use of mysql_link_fields a bit
+Fills STMT's lengths array for given row. Makes use of myodbc_link_fields a bit
 less terrible.
 
 @param[in,out] stmt     The statement to modify
@@ -474,8 +474,8 @@ copy_ansi_result(STMT *stmt,
   SQLCHAR *result_end;
   ulong used_bytes= 0, used_chars= 0, error_count= 0;
 
-  my_bool convert_binary= test(field->charsetnr == BINARY_CHARSET_NUMBER) &&
-                          test(field->org_table_length == 0) &&
+  my_bool convert_binary= (field->charsetnr == BINARY_CHARSET_NUMBER ? 1 : 0) &&
+                          (field->org_table_length == 0 ? 1 : 0) &&
                           stmt->dbc->ds->handle_binary_as_char;
 
   CHARSET_INFO *to_cs= stmt->dbc->ansi_charset_info,
@@ -995,8 +995,8 @@ SQLRETURN copy_binhex_result(STMT *stmt,
 */
 SQLSMALLINT get_sql_data_type(STMT *stmt, MYSQL_FIELD *field, char *buff)
 {
-  my_bool field_is_binary= test(field->charsetnr == BINARY_CHARSET_NUMBER) &&
-                           (test(field->org_table_length > 0) ||
+  my_bool field_is_binary= (field->charsetnr == BINARY_CHARSET_NUMBER ? 1 : 0) &&
+                           ((field->org_table_length > 0 ? 1 : 0) ||
                             !stmt->dbc->ds->handle_binary_as_char);
 
   switch (field->type) {
@@ -1339,8 +1339,8 @@ SQLULEN get_column_size(STMT *stmt, MYSQL_FIELD *field)
   case MYSQL_TYPE_DECIMAL:
   case MYSQL_TYPE_NEWDECIMAL:
     return (length -
-            test(!(field->flags & UNSIGNED_FLAG)) - /* sign? */
-            test(field->decimals));                 /* decimal point? */
+            (!(field->flags & UNSIGNED_FLAG) ? 1:  0) - /* sign? */
+            (field->decimals ? 1 : 0));             /* decimal point? */
 
   case MYSQL_TYPE_BIT:
     /*
@@ -1536,16 +1536,16 @@ SQLLEN get_display_size(STMT *stmt __attribute__((unused)),MYSQL_FIELD *field)
 
   switch (field->type) {
   case MYSQL_TYPE_TINY:
-    return 3 + test(field->flags & UNSIGNED_FLAG);
+    return 3 + (field->flags & UNSIGNED_FLAG ? 1 : 0);
 
   case MYSQL_TYPE_SHORT:
-    return 5 + test(field->flags & UNSIGNED_FLAG);
+    return 5 + (field->flags & UNSIGNED_FLAG ? 1 : 0);
 
   case MYSQL_TYPE_INT24:
-    return 8 + test(field->flags & UNSIGNED_FLAG);
+    return 8 + (field->flags & UNSIGNED_FLAG ? 1 : 0);
 
   case MYSQL_TYPE_LONG:
-    return 10 + test(field->flags & UNSIGNED_FLAG);
+    return 10 + (field->flags & UNSIGNED_FLAG ? 1 : 0);
 
   case MYSQL_TYPE_FLOAT:
     return 14;
@@ -2284,6 +2284,16 @@ my_bool reget_current_catalog(DBC *dbc)
 
 int myodbc_strcasecmp(const char *s, const char *t)
 {
+  if (!s && !t)
+  {
+    return 0;
+  }
+
+  if (!s || !t)
+  {
+    return 1;
+  }
+
   while (toupper((uchar) *s) == toupper((uchar) *t++))
     if (!*s++)
       return 0;
@@ -2298,6 +2308,16 @@ int myodbc_strcasecmp(const char *s, const char *t)
 
 int myodbc_casecmp(const char *s, const char *t, uint len)
 {
+  if (!s && !t)
+  {
+    return 0;
+  }
+
+  if (!s || !t)
+  {
+    return (int)len + 1;
+  }
+
   while (len-- != 0 && toupper(*s++) == toupper(*t++))
     ;
   return (int)len + 1;
@@ -2491,7 +2511,7 @@ ulong myodbc_escape_string(MYSQL *mysql __attribute__((unused)),
         overflow= TRUE;
         break;
       }
-      *to++= '\\';
+      *to++= escape != '`' ? '\\' : '`';
       *to++= escape;
     }
     else
@@ -3745,7 +3765,7 @@ char *extend_buffer(NET *net, char *to, ulong length)
     need= (ulong)(to - (char *)net->buff) + length;
     if (!to || need > net->max_packet - 10)
     {
-        if (net_realloc(net, need))
+        if (myodbc_net_realloc(net, need))
         {
             return 0;
         }
@@ -3820,4 +3840,110 @@ BOOL got_out_parameters(STMT *stmt)
   }
 
   return FALSE;
-}
\ No newline at end of file
+}
+
+
+int get_session_variable(STMT *stmt, const char *var, char *result)
+{
+  char buff[255+4*NAME_CHAR_LEN], *to;
+  MYSQL_RES *res;
+  MYSQL_ROW row;
+
+  if (var)
+  {
+    to= strmov(buff, "SHOW SESSION VARIABLES LIKE '");
+    to= strmov(to, var);
+    to= strmov(to, "'");
+    *to= '\0';
+
+    if (!SQL_SUCCEEDED(odbc_stmt(stmt->dbc, buff)))
+    {
+      return 0;
+    }
+
+    res= mysql_store_result(&stmt->dbc->mysql);
+    if (!res)
+      return 0;
+
+    row= mysql_fetch_row(res);
+    if (row)
+    {
+      strcpy(result, row[1]);
+      mysql_free_result(res);
+      return strlen(result);
+    }
+  }
+
+  return 0;
+}
+
+
+const char get_identifier_quote(STMT *stmt)
+{
+  const char tick= '`', quote= '"', empty= ' ';
+
+  if (is_minimum_version(stmt->dbc->mysql.server_version, "3.23.06"))
+  {
+    /* 
+      The full list of all SQL modes takes over 512 symbols, so we reserve
+      some for the future
+     */
+    char sql_mode[2048]= {0};
+    uint length= get_session_variable(stmt, "SQL_MODE", (char*)sql_mode);
+    
+    const char *end=  sql_mode + length;
+    if (find_first_token(stmt->dbc->ansi_charset_info, sql_mode, end, "ANSI_QUOTES"))
+    {
+      return quote;
+    }
+    else
+    {
+      return tick;
+    }
+  }
+  return empty;
+}
+
+
+/** Realloc the NET packet buffer. */
+my_bool myodbc_net_realloc(NET *net, size_t length)
+{
+  uchar *buff;
+  size_t pkt_length;
+
+  if (length >= net->max_packet_size)
+  {
+    /* @todo: 1 and 2 codes are identical. */
+    net->error= 1;
+    net->last_errno= ER_NET_PACKET_TOO_LARGE;
+    return 1;
+  }
+  pkt_length = (length+IO_SIZE-1) & ~(IO_SIZE-1); 
+  /*
+    We must allocate some extra bytes for the end 0 and to be able to
+    read big compressed blocks + 1 safety byte since uint3korr() in
+    net_read_packet() may actually read 4 bytes depending on build flags and
+    platform.
+  */
+  if (!(buff= (uchar*) my_realloc((char*) net->buff, pkt_length +
+                                  NET_HEADER_SIZE + COMP_HEADER_SIZE + 1,
+                                  MYF(MY_WME))))
+  {
+    /* @todo: 1 and 2 codes are identical. */
+    net->error= 1;
+    net->last_errno= ER_OUT_OF_RESOURCES;
+    /* In the server the error is reported by MY_WME flag. */
+    return 1;
+  }
+  net->buff=net->write_pos=buff;
+  net->buff_end=buff+(net->max_packet= (ulong) pkt_length);
+  return 0;
+}
+
+
+/** Free net packet buffer. */
+void myodbc_net_end(NET *net)
+{
+  my_free(net->buff);
+  net->buff=0;
+}
