@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2007, 2013, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2007, 2014, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -367,6 +367,8 @@ void syncTabsData(HWND hwnd, DataSource &params)
   GET_BOOL(MISC_TAB, read_options_from_mycnf);
   GET_BOOL(MISC_TAB, disable_transactions);
   GET_BOOL(MISC_TAB, min_date_to_zero);
+  GET_BOOL(MISC_TAB, no_ssps);
+  GET_BOOL(MISC_TAB, default_bigint_bind_str);
 }
 
 
@@ -452,6 +454,8 @@ void syncTabs(HWND hwnd, DataSource &params)
   SET_BOOL(MISC_TAB, read_options_from_mycnf);
   SET_BOOL(MISC_TAB, disable_transactions);
   SET_BOOL(MISC_TAB, min_date_to_zero);
+  SET_BOOL(MISC_TAB, no_ssps);
+  SET_BOOL(MISC_TAB, default_bigint_bind_str);
 }
 
 
