@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2003, 2013, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2003, 2014, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/ODBC is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most
@@ -243,13 +243,17 @@ DECLARE_TEST(t_bug19823)
                                   &timeout, 0, NULL));
   is_num(timeout, 17);
 
+#ifndef USE_IODBC
   /*
     SQL_ATTR_CONNECTION_TIMEOUT is always 0, because the driver does not
     support it and the driver just silently swallows any value given for it.
+    Also, iODBC returns error for SQL_ATTR_CONNECTION_TIMEOUT requested if
+    the connection is established.
   */
   ok_con(hdbc1, SQLGetConnectAttr(hdbc1, SQL_ATTR_CONNECTION_TIMEOUT,
                                   &timeout, 0, NULL));
   is_num(timeout, 0);
+#endif
 
   ok_con(hdbc1, SQLDisconnect(hdbc1));
   ok_con(hdbc1, SQLFreeConnect(hdbc1));
@@ -683,6 +687,7 @@ DECLARE_TEST(sqlcancel)
 {
   pthread_t thread;
 
+#ifdef IODBC_BUG_SQLCANCEL_FIXED
   pthread_create(&thread, NULL, cancel_in_one_second, hstmt);
 
   /* SLEEP(n) returns 1 when it is killed. */
@@ -691,6 +696,7 @@ DECLARE_TEST(sqlcancel)
   is_num(my_fetch_int(hstmt, 1), 1);
 
   pthread_join(thread, NULL);
+#endif // ifdef IODBC_BUG_SQLCANCEL_FIXED
 
   return OK;
 }
@@ -1013,7 +1019,7 @@ DECLARE_TEST(t_bug45378)
 
 
 /*
-  Bug#45378 - Crash in SQLSetConnectAttr
+  Bug#63844 - Crash in SQLSetConnectAttr
 */
 DECLARE_TEST(t_bug63844)
 {
