@@ -23,6 +23,7 @@
 */
 
 #include "odbctap.h"
+#include "../VersionInfo.h"
 
 DECLARE_TEST(my_columns_null)
 {
@@ -582,7 +583,7 @@ typedef struct t_table_bug
 } t_describe_col;
 
 
-t_describe_col t_tables_bug_data[5] =
+t_describe_col t_tables_bug_data_unicode[5] =
 {
   {"TABLE_CAT",   9, SQL_WVARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
   {"TABLE_SCHEM",11, SQL_WVARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
@@ -591,12 +592,24 @@ t_describe_col t_tables_bug_data[5] =
   {"REMARKS",     7, SQL_WVARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
 };
 
+t_describe_col t_tables_bug_data_ansi[5] =
+{
+  {"TABLE_CAT",   9, SQL_VARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
+  {"TABLE_SCHEM",11, SQL_VARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
+  {"TABLE_NAME", 10, SQL_VARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
+  {"TABLE_TYPE", 10, SQL_VARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
+  {"REMARKS",     7, SQL_VARCHAR, MYSQL_NAME_LEN, 0, SQL_NULLABLE},
+
+};
+
 
 DECLARE_TEST(t_tables_bug)
 {
   SQLSMALLINT i, ColumnCount, pcbColName, pfSqlType, pibScale, pfNullable;
   SQLULEN     pcbColDef;
   SQLCHAR     szColName[MAX_NAME_LEN];
+  t_describe_col *t_tables_bug_data= unicode_driver ? t_tables_bug_data_unicode
+                                                    : t_tables_bug_data_ansi;
 
   ok_stmt(hstmt,  SQLTables(hstmt, NULL, 0, NULL, 0, NULL, 0,
                             (SQLCHAR *)"'TABLE'", SQL_NTS));
@@ -610,13 +623,13 @@ DECLARE_TEST(t_tables_bug)
                                  MAX_NAME_LEN, &pcbColName, &pfSqlType,
                                  &pcbColDef, &pibScale, &pfNullable));
 
-    fprintf(stdout, "# Column '%d':\n", i);
-    fprintf(stdout, "#  Column Name   : %s\n", szColName);
-    fprintf(stdout, "#  NameLengh     : %d\n", pcbColName);
-    fprintf(stdout, "#  DataType      : %d\n", pfSqlType);
-    fprintf(stdout, "#  ColumnSize    : %d\n", pcbColDef);
-    fprintf(stdout, "#  DecimalDigits : %d\n", pibScale);
-    fprintf(stdout, "#  Nullable      : %d\n", pfNullable);
+    printMessage("Column '%d':", i);
+    printMessage(" Column Name   : %s", szColName);
+    printMessage(" NameLengh     : %d", pcbColName);
+    printMessage(" DataType      : %d", pfSqlType);
+    printMessage(" ColumnSize    : %d", pcbColDef);
+    printMessage(" DecimalDigits : %d", pibScale);
+    printMessage(" Nullable      : %d", pfNullable);
 
     is_str(t_tables_bug_data[i-1].szColName, szColName, pcbColName);
     is_num(t_tables_bug_data[i-1].pcbColName, pcbColName);
@@ -636,52 +649,117 @@ DECLARE_TEST(t_tables_bug)
 }
 
 
-DECLARE_TEST(t_current_catalog)
+DECLARE_TEST(t_current_catalog_unicode)
+{
+  SQLWCHAR    db[255];
+  wchar_t     cur_db[255];
+  SQLRETURN   rc;
+  SQLINTEGER  len;
+
+  REQUIRES_UNICODE_DRIVER;
+
+  rc = SQLFreeStmt(hstmt,SQL_CLOSE);
+  mystmt(hstmt,rc);
+
+  rc = SQLGetConnectAttrW(hdbc, SQL_ATTR_CURRENT_CATALOG, db, sizeof(db), &len);
+  mycon(hdbc,rc);
+
+  is_num(len, strlen("test") * sizeof(SQLWCHAR));
+  is_wstr(sqlwchar_to_wchar_t(db), L"test", 5);
+
+  rc = SQLSetConnectAttrW(hdbc, SQL_ATTR_CURRENT_CATALOG, db, SQL_NTS);
+  mycon(hdbc,rc);
+
+  ok_sql(hstmt, "DROP DATABASE IF EXISTS test_odbc_current");
+
+  wcscpy(cur_db, L"test_odbc_current");
+  rc = SQLSetConnectAttrW(hdbc, SQL_ATTR_CURRENT_CATALOG, W(cur_db), SQL_NTS);
+  mycon_r(hdbc,rc);
+
+  ok_sql(hstmt, "CREATE DATABASE test_odbc_current");
+  rc = SQLFreeStmt(hstmt,SQL_CLOSE);
+
+  wcscpy(cur_db, L"test_odbc_current");
+  rc = SQLSetConnectAttrW(hdbc, SQL_ATTR_CURRENT_CATALOG, W(cur_db), SQL_NTS);
+  mycon(hdbc,rc);
+
+  rc = SQLGetConnectAttrW(hdbc, SQL_ATTR_CURRENT_CATALOG, db, 255, &len);
+  mycon(hdbc,rc);
+
+  is_num(len, strlen("test_odbc_current") * sizeof(SQLWCHAR));
+  is_wstr(sqlwchar_to_wchar_t(db), cur_db, 18);
+
+  wcscpy(cur_db, L"test_odbc_current_12455");
+  rc = SQLSetConnectAttrW(hdbc, SQL_ATTR_CURRENT_CATALOG, W(cur_db), SQL_NTS);
+  mycon_r(hdbc,rc);
+
+#ifdef _WIN32
+  rc = SQLSetConnectAttrW(hdbc, SQL_ATTR_CURRENT_CATALOG, W(cur_db), len);
+#else
+  rc = SQLSetConnectAttrW(hdbc, SQL_ATTR_CURRENT_CATALOG, W(cur_db), len / sizeof(SQLWCHAR));
+#endif
+  mycon(hdbc,rc);
+
+  /* reset for further tests */
+  wcscpy(cur_db, L"test");
+  rc = SQLSetConnectAttrW(hdbc, SQL_ATTR_CURRENT_CATALOG, W(cur_db), SQL_NTS);
+  mycon(hdbc,rc);
+
+  ok_sql(hstmt, "DROP DATABASE test_odbc_current");
+
+  return OK;
+}
+
+
+DECLARE_TEST(t_current_catalog_ansi)
 {
   SQLCHAR     cur_db[255], db[255];
   SQLRETURN   rc;
   SQLINTEGER len;
 
-    rc = SQLFreeStmt(hstmt,SQL_CLOSE);
-    mystmt(hstmt,rc);
+  REQUIRES_ANSI_DRIVER;
 
-    rc = SQLGetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, db, sizeof(db), &len);
-    mycon(hdbc,rc);
-    fprintf(stdout,"current_catalog: %s (%ld)\n", db, len);
-    is_num(len, 4);
-    is_str(db, "test", 5);
+  rc = SQLFreeStmt(hstmt,SQL_CLOSE);
+  mystmt(hstmt,rc);
 
-    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, db, SQL_NTS);
-    mycon(hdbc,rc);
+  rc = SQLGetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, db, sizeof(db), &len);
+  mycon(hdbc,rc);
 
-    ok_sql(hstmt, "DROP DATABASE IF EXISTS test_odbc_current");
+  is_num(len, 4);
+  is_str(db, "test", 5);
 
-    strcpy((char *)cur_db, "test_odbc_current");
-    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, cur_db, SQL_NTS);
-    mycon_r(hdbc,rc);
+  rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, db, SQL_NTS);
+  mycon(hdbc,rc);
 
-    ok_sql(hstmt, "CREATE DATABASE test_odbc_current");
+  ok_sql(hstmt, "DROP DATABASE IF EXISTS test_odbc_current");
 
-    strcpy((char *)cur_db, "test_odbc_current");
-    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, cur_db, SQL_NTS);
-    mycon(hdbc,rc);
+  strcpy((char *)cur_db, "test_odbc_current");
+  rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, cur_db, SQL_NTS);
+  mycon_r(hdbc,rc);
 
-    rc = SQLGetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, db, 255, &len);
-    mycon(hdbc,rc);
-    fprintf(stdout,"current_catalog: %s (%ld)\n", db, len);
-    is_num(len, 17);
-    is_str(db, cur_db, 18);
+  ok_sql(hstmt, "CREATE DATABASE test_odbc_current");
+  rc = SQLFreeStmt(hstmt,SQL_CLOSE);
 
-    strcpy((char *)cur_db, "test_odbc_current_12455");
-    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, cur_db, SQL_NTS);
-    mycon_r(hdbc,rc);
+  strcpy((char *)cur_db, "test_odbc_current");
+  rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, cur_db, SQL_NTS);
+  mycon(hdbc,rc);
 
-    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, cur_db, len);
-    mycon(hdbc,rc);
+  rc = SQLGetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, db, 255, &len);
+  mycon(hdbc,rc);
 
-    /* reset for further tests */
-    rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, (SQLCHAR *)"test", SQL_NTS);
-    mycon(hdbc,rc);
+  is_num(len, 17);
+  is_str(db, cur_db, 18);
+
+  strcpy((char *)cur_db, "test_odbc_current_12455");
+  rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, cur_db, SQL_NTS);
+  mycon_r(hdbc,rc);
+
+  rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, cur_db, len);
+  mycon(hdbc,rc);
+
+  /* reset for further tests */
+  rc = SQLSetConnectAttr(hdbc, SQL_ATTR_CURRENT_CATALOG, (SQLCHAR *)"test", SQL_NTS);
+  mycon(hdbc,rc);
 
   ok_sql(hstmt, "DROP DATABASE test_odbc_current");
 
@@ -792,11 +870,7 @@ DECLARE_TEST(t_sqltables)
 
 DECLARE_TEST(my_information_schema)
 {
-  SQLCHAR   conn[512], conn_out[512];
-  HDBC hdbc1;
-  HSTMT hstmt1;
-  SQLSMALLINT conn_out_len;
-  SQLRETURN rc;
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
 
   ok_sql(hstmt, "DROP DATABASE IF EXISTS istest__");
   ok_sql(hstmt, "DROP DATABASE IF EXISTS istest_1");
@@ -811,43 +885,21 @@ DECLARE_TEST(my_information_schema)
   ok_sql(hstmt, "CREATE TABLE istest_2.istab2(a INT,b INT,c INT, d INT)");
 
   /* We need to have istest__ as the default DB */
-  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s;DATABASE=istest__;OPTION=0", mydsn, myuid, mypwd);
-  if (mysock != NULL)
-  {
-    strcat((char *)conn, ";SOCKET=");
-    strcat((char *)conn, (char *)mysock);
-  }
-  if (myport)
-  {
-    char pbuff[20];
-    sprintf(pbuff, ";PORT=%d", myport);
-    strcat((char *)conn, pbuff);
-  }
-
-  ok_env(henv, SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc1));
-
-  ok_con(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, sizeof(conn), conn_out,
-                                 sizeof(conn_out), &conn_out_len,
-                                 SQL_DRIVER_NOPROMPT));
-  ok_con(hdbc1, SQLAllocStmt(hdbc1, &hstmt1));
-
-  rc = SQLTables(hstmt1, "istest__", SQL_NTS, "", 0, "istab%", SQL_NTS, NULL, 0);
-  mystmt(hstmt1,rc);
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+                                        NULL, NULL, "istest__", NULL));
+  
+  ok_stmt(hstmt1, SQLTables(hstmt1, "istest__", SQL_NTS, "", 0, "istab%",
+                            SQL_NTS, NULL, 0));
 
   /* all tables from all databases should be displayed */
   is_num(my_print_non_format_result(hstmt1), 3);
-  rc = SQLFreeStmt(hstmt1, SQL_CLOSE);
+  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_CLOSE));
 
-  rc = SQLTables(hstmt1, NULL, 0, NULL, 0, "istab%", SQL_NTS, NULL, 0);
-  mystmt(hstmt1,rc);
+  ok_stmt(hstmt1, SQLTables(hstmt1, NULL, 0, NULL, 0, "istab%", SQL_NTS, NULL, 0));
 
   is_num(my_print_non_format_result(hstmt1), 1);
-  rc = SQLFreeStmt(hstmt1, SQL_CLOSE);
-  mystmt(hstmt1,rc);
-
-  ok_con(hdbc1, SQLDisconnect(hdbc1));
-  ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
 
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
   return OK;
 }
 
@@ -1049,34 +1101,16 @@ DECLARE_TEST(t_bug23031)
 */
 DECLARE_TEST(bug15713)
 {
-  HDBC hdbc1;
-  HSTMT hstmt1;
-  SQLCHAR   conn[512], conn_out[512];
-  SQLSMALLINT conn_out_len;
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
+  SQLCHAR   conn[512];
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug15713");
   ok_sql(hstmt, "CREATE TABLE t_bug15713 (a INT)");
 
   /* The connection strings must not include DATABASE. */
-  sprintf((char *)conn, "DSN=%s;UID=%s;PWD=%s", mydsn, myuid, mypwd);
-  if (mysock != NULL)
-  {
-    strcat((char *)conn, ";SOCKET=");
-    strcat((char *)conn, (char *)mysock);
-  }
-  if (myport)
-  {
-    char pbuff[20];
-    sprintf(pbuff, ";PORT=%d", myport);
-    strcat((char *)conn, pbuff);
-  }
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL,
+                                        NULL, NULL, "", NULL));
 
-  ok_env(henv, SQLAllocHandle(SQL_HANDLE_DBC, henv, &hdbc1));
-
-  ok_con(hdbc1, SQLDriverConnect(hdbc1, NULL, conn, sizeof(conn), conn_out,
-                                 sizeof(conn_out), &conn_out_len,
-                                 SQL_DRIVER_NOPROMPT));
-  ok_con(hdbc1, SQLAllocStmt(hdbc1, &hstmt1));
 
   ok_stmt(hstmt1, SQLColumns(hstmt1, (SQLCHAR *)"test", SQL_NTS,
                              NULL, 0, (SQLCHAR *)"t_bug15713", SQL_NTS,
@@ -1089,9 +1123,7 @@ DECLARE_TEST(bug15713)
 
   expect_stmt(hstmt1, SQLFetch(hstmt1), SQL_NO_DATA_FOUND);
 
-  ok_stmt(hstmt1, SQLFreeStmt(hstmt1, SQL_DROP));
-  ok_con(hdbc1, SQLDisconnect(hdbc1));
-  ok_con(hdbc1, SQLFreeHandle(SQL_HANDLE_DBC, hdbc1));
+  free_basic_handles(&henv1, &hdbc1, &hstmt1);
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS t_bug15713");
   return OK;
@@ -1179,11 +1211,9 @@ DECLARE_TEST(bug8860)
 */
 DECLARE_TEST(t_bug26934)
 {
-  HENV henv1;
-  HDBC hdbc1;
-  HSTMT hstmt1;
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
 
-  alloc_basic_handles(&henv1, &hdbc1, &hstmt1);
+  is(OK == alloc_basic_handles(&henv1, &hdbc1, &hstmt1));
 
   ok_sql(hstmt1, "SET @@wait_timeout = 1");
   sleep(2);
@@ -1385,17 +1415,14 @@ DECLARE_TEST(t_bug33298)
 */
 DECLARE_TEST(t_bug12805)
 {
-  SQLHENV     henv1;
-  SQLHDBC     hdbc1;
-  SQLHSTMT    hstmt1;
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
   SQLCHAR     dummy[10];
   SQLULEN     length;  
   SQLUINTEGER len2;
-  int         saved_options= myoption;
 
-  SET_DSN_OPTION(myoption | 1 << 27);
-
-  alloc_basic_handles(&henv1, &hdbc1, &hstmt1);
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, NULL, 
+                                        NULL, NULL, NULL, 
+                                        "COLUMN_SIZE_S32=1"));
 
   ok_sql(hstmt1, "DROP TABLE IF EXISTS bug12805");
   ok_sql(hstmt1, "CREATE TABLE bug12805("\
@@ -1442,7 +1469,6 @@ DECLARE_TEST(t_bug12805)
   ok_stmt(hstmt, SQLFreeStmt(hstmt, SQL_CLOSE));
   ok_sql(hstmt, "DROP TABLE bug12805");
 
-  SET_DSN_OPTION(saved_options);
   return OK;
 }
 
@@ -1452,32 +1478,15 @@ DECLARE_TEST(t_bug12805)
 */
 DECLARE_TEST(t_bug30770)
 {
+  DECLARE_BASIC_HANDLES(henv1, hdbc1, hstmt1);
   SQLCHAR    buff[512];
-  SQLHENV    henv1;
-  SQLHDBC    hdbc1;
-  SQLHSTMT   hstmt1;
-  SQLCHAR    conn[MAX_NAME_LEN*2];
-
 
   ok_sql(hstmt, "DROP TABLE IF EXISTS bug30770");
   ok_sql(hstmt, "CREATE TABLE bug30770 (a INT)");
 
   /* Connect with no default daabase */
-  sprintf((char *)conn, "DRIVER=%s;SERVER=%s;" \
-                        "UID=%s;PASSWORD=%s;DATABASE=%s", mydriver, myserver,
-                        myuid, mypwd, mydb);
-  if (mysock != NULL)
-  {
-    strcat((char *)conn, ";SOCKET=");
-    strcat((char *)conn, (char *)mysock);
-  }
-  if (myport)
-  {
-    char pbuff[20];
-    sprintf(pbuff, ";PORT=%d", myport);
-    strcat((char *)conn, pbuff);
-  }
-  is_num(mydrvconnect(&henv1, &hdbc1, &hstmt1, conn), OK);
+  is(OK == alloc_basic_handles_with_opt(&henv1, &hdbc1, &hstmt1, USE_DRIVER,
+                                        NULL, NULL, "", NULL));
 
   /* Get the info from just one table.  */
   ok_stmt(hstmt1, SQLColumns(hstmt1, NULL, SQL_NTS, NULL, SQL_NTS,
@@ -1553,7 +1562,8 @@ BEGIN_TESTS
   ADD_TEST(tmysql_specialcols)
   ADD_TEST(t_columns)
   ADD_TEST(t_tables_bug)
-  ADD_TEST(t_current_catalog)
+  ADD_TEST(t_current_catalog_unicode)
+  ADD_TEST(t_current_catalog_ansi)
   ADD_TEST(tmysql_showkeys)
   ADD_TEST(t_sqltables)
   ADD_TEST(my_information_schema)
