@@ -234,9 +234,9 @@ void mem_gc_init()
   gc_blk.counter= 0;
 }
 
-#define DECLARE_BASIC_HANDLES(E, C, S) SQLHENV E; \
-  SQLHDBC C; \
-  SQLHSTMT S
+#define DECLARE_BASIC_HANDLES(E, C, S) SQLHENV E= NULL; \
+  SQLHDBC C= NULL; \
+  SQLHSTMT S= NULL
 
 #define BEGIN_TESTS my_test tests[]= {
 #define ADD_TEST(name)          { #name, name, OK  , -1 },
@@ -1165,11 +1165,22 @@ int get_connection(SQLHDBC *hdbc, const SQLCHAR *dsn, const SQLCHAR *uid,
   /* Buffers have to be large enough to contain SSL options and long names */
   SQLCHAR     connIn[4096], connOut[4096];
   SQLCHAR     dsn_buf[MAX_NAME_LEN]= {0}, socket_buf[MAX_NAME_LEN]= {0};
-  SQLCHAR     db_buf[MAX_NAME_LEN]= {0}, port_buf[MAX_NAME_LEN]= {0};
+  /* ";database="+ we make buffer bigger for one certain test */
+  SQLCHAR     db_buf[4096]= {0};
+  /* Should fit 8 byte + ";port=" */
+  SQLCHAR     port_buf[32]= {0};
   SQLSMALLINT len;
   SQLRETURN   rc;
   SQLCHAR     driver_name[16]; /* Should be enough for myodbc library file name */
 
+  /* We never set the custom DSN, but sometimes use DRIVER instead */
+  if (dsn == NULL)
+    sprintf((char *)dsn_buf, "DSN=%s", (char *)mydsn);
+  else if (dsn == USE_DRIVER)
+    sprintf((char *)dsn_buf, "DRIVER=%s", (char *)mydriver);
+  else
+    sprintf((char *)dsn_buf, "DSN=%s", (char *)dsn);
+
   /* We never set the custom DSN, but sometimes use DRIVER instead */
   if (dsn == NULL)
     sprintf((char *)dsn_buf, "DSN=%s", (char *)mydsn);
