@@ -286,17 +286,17 @@ Driver *driver_new()
                                      MYF(0));
   if (!driver->lib)
   {
-    x_free(driver);
     x_free(driver->name);
+    x_free(driver);
     return NULL;
   }
   driver->setup_lib= (SQLWCHAR *)my_malloc(ODBCDRIVER_STRLEN *
                                            sizeof(SQLWCHAR), MYF(0));
   if (!driver->setup_lib)
   {
-    x_free(driver);
     x_free(driver->name);
     x_free(driver->lib);
+    x_free(driver);
     return NULL;
   }
   /* init to empty strings */
@@ -524,6 +524,12 @@ int driver_from_kvpair_semicolon(Driver *driver, const SQLWCHAR *attrs)
     if ((end= sqlwcharchr(attrs, ';')) == NULL)
       end= attrs + sqlwcharlen(attrs);
 
+    /* check the attribute length (must not be longer than the buffer size) */
+    if (split - attrs >= 100)
+    {
+      return 1;
+    }
+
     /* pull out the attribute name */
     memcpy(attribute, attrs, (split - attrs) * sizeof(SQLWCHAR));
     attribute[split - attrs]= 0; /* add null term */
@@ -539,6 +545,14 @@ int driver_from_kvpair_semicolon(Driver *driver, const SQLWCHAR *attrs)
 
     if (dest)
     {
+      if (end - split >= ODBCDRIVER_STRLEN)
+      {
+        /* 
+          The value is longer than the allocated buffer length
+          for driver->lib or driver->setup_lib
+        */
+        return 1;
+      }
       memcpy(dest, split, (end - split) * sizeof(SQLWCHAR));
       dest[end - split]= 0; /* add null term */
     }
@@ -956,6 +970,13 @@ int ds_from_kvpair(DataSource *ds, const SQLWCHAR *attrs, SQLWCHAR delim)
     while (*attrs == ' ')
       ++attrs;
     len = split - attrs;
+
+    /* the attribute length must not be longer than the buffer size */
+    if (len >= 100)
+    {
+      return 1;
+    }
+
     memcpy(attribute, attrs, len * sizeof(SQLWCHAR));
     attribute[len]= 0;
     /* remove trailing spaces on attribute */
